clear all;
format long;

%---------------------------------------------------------------------------------------
% This code plots Pd vs. Pf for a given SNR by varying the detection threshold
%---------------------------------------------------------------------------------------
%                                   coherent mode
%   a. N    = 31;       %coherent average length (code length or a multiple of code length)
%   b. M    = 62;       %coherent average length (code length or a multiple of code length)
%   b. L    = 15        %number of M-Length codes (not pulses) used in the noncoherent average
%                       %M-length code = integer concatenation of basic code, in this case 2 codes
%   c. GbdB = 2.4       %estimated EbN0 (dB) for a coherent radio
%---------------------------------------------------------------------------------------

%parameters
%==========
N       = 31;                           %code length
M       = N*1;                          %coherent average length (integer multiple of code length)
L       = 1;                            %noncoherent average of L supercodes (integer multiple of code)
alpha   = 1;                            %fraction of multipath energy captured in main path
GbdB    = 13;                           %EbN0 per symbol or code (dB)
margin  = 0;                            %margin to account for implementation losses
GpdB    = GbdB - 10*log10(N) - margin;  %EpN0 per pulse (dB) (average)
Gp      = 10^(GpdB/10);                 %EpN0 per pulse (Linear)

nn      = round(log2(N));               %log2 of code length
Tmax    = 4*L*M*Gp;                     %maximum threshold value
dT      = Tmax/256;                     %threshold step
Tv      = 1e-3:dT:Tmax;LT = length(Tv); %vary threshold from zero to maximum value
PD      = zeros(1,LT);
PF      = zeros(1,LT);
i = 0;Flag = 0;
while ((i < LT) & (Flag == 0));
    i = i + 1;T = Tv(i);
    PD(i) = wbaMarcumq(sqrt(2*L*M*Gp*alpha),sqrt(2*T),L);
    LogT = log(T);LogX = -T;Z = exp(-T);
    for k = 1:L-1, LogX = LogX + LogT - log(k);Z = Z + exp(LogX);end;PF(i) = Z;
    for m = 1:nn, PF(i) = 2*PF(i) - PF(i)*PF(i);end;
    if (PF(i) < 1e-3), Flag = 1;end;
    fprintf('i = %d, Threshold = %f, Pd = %f, Pf = %f\n',i, T,PD(i),PF(i));
end;

figure;plot(PF,PD);
grid;xlabel('False alarm probability P_F');ylabel('Detection probability P_D');
title('PD vs. PF');axis([0 0.25 0.75 1]);